@@ -4,22 +4,28 @@
 #' wavebands of a radiation spectrum.
 #'
 #' @param w.length numeric Vector of wavelengths (nm)
-#' @param s.irrad numeric Corresponding vector of spectral (energy) irradiances
-#'   (W m-2 nm-1)
-#' @param w.band.num waveband
-#' @param w.band.denom waveband
+#' @param s.irrad numeric vector of spectral (energy or photon) irradiances (W
+#'   m-2 nm-1) or (mol s-1 m-2 nm-1).
+#' @param w.band.num waveband object used to compute the numerator of the ratio.
+#' @param w.band.denom waveband object used to compute the denominator of the
+#'   ratio.
 #' @param unit.out.num character Allowed values "energy", and "photon", or its
-#'   alias "quantum"
+#'   alias "quantum".
 #' @param unit.out.denom character Allowed values "energy", and "photon", or its
-#'   alias "quantum"
+#'   alias "quantum".
 #' @param unit.in character Allowed values "energy", and "photon", or its alias
-#'   "quantum"
+#'   "quantum".
 #' @param check.spectrum logical Flag indicating whether to sanity check input
-#'   data, default is TRUE
+#'   data, default is TRUE.
 #' @param use.cached.mult logical Flag indicating whether multiplier values
-#'   should be cached between calls
-#' @param use.hinges logical Flag indicating whether to use hinges to reduce
-#'   interpolation errors
+#'   should be cached between calls.
+#' @param use.hinges logical Flag indicating whether to insert "hinges" into the
+#'   spectral data before integration so as to reduce interpolation errors at
+#'   the boundaries of the wavebands.
+#'
+#' @note The default for both \code{w.band} parameters is a waveband covering
+#'   the whole range of \code{w.length}. From version 9.19 onwards use of this
+#'   default does not trigger a warning, but instead is used silently.
 #'
 #' @return a single numeric value giving the ratio
 #'
@@ -59,44 +65,40 @@ waveband_ratio <-
            check.spectrum = TRUE,
            use.cached.mult = FALSE,
            use.hinges = getOption("photobiology.use.hinges",
-                                  default = NULL) ) {
+                                  default = NULL)) {
     # We duplicate code from irradiance() here to avoid repeated checks
     # and calculations on the same data
     #
     # what output? seems safer to not have a default here
     if (is.null(unit.out.num) || is.null(unit.out.denom)) {
       warning("'unit.out.num' has no default value")
-      return(NA)
+      return(NA_real_)
     }
     # make code a bit simpler further down
     if (unit.in == "quantum") {unit.in <- "photon"}
     # sanity check for wavelengths
     if (check.spectrum && !check_spectrum(w.length, s.irrad)) {
-      return(NA)
+      return(NA_real_)
     }
     # if the waveband for numerator is undefined then use
     # the whole wavelength range of the spectrum for numerator
     if (is.null(w.band.num)) {
       w.band.num <- new_waveband(min(w.length),max(w.length))
-      warning("'w.band.num' not supplied, using whole range of data instead.")
     }
     # if the waveband for denominator is undefined then use
     # the whole wavelength range of the spectrum for denominator
     if (is.null(w.band.denom)) {
       w.band.denom <- new_waveband(min(w.length),max(w.length))
-      warning("'w.band.denom' not supplied, using whole range of data instead.")
     }
     # choose whether to use hinges or not
     # if the user has specified its value, we leave it alone
     # but if it was not requested, we decide whether to use
     # it or not based of the wavelength resolution of the
-    # spectrum. This will produce small errors for high
-    # spectral resulution data, and speed up the calculations
-    # a lot in such cases
+    # spectrum.
     if (is.null(use.hinges)) {
       use.hinges <- auto_hinges(w.length)
     }
-    # if the w.band.num and/or w.band.denom include 'hinges' we insert them
+    # if the w.band.num and/or w.band.denom include 'hinges' we insert them.
     # it is o.k. to have hinges unsorted!
     # in new_waveband() NULL hinges are replaced with numeric(0)
     if (use.hinges) {
